import Image from "next/image";
import { Dispatch, SetStateAction, useMemo, useState } from "react";
import { Button, InputGroup } from "react-bootstrap";
import { TbLayoutGrid, TbLayoutList } from "react-icons/tb";

import { BookDataRet } from "@snip/database/types";

import { DropdownInput, DropdownItem } from "components/form/inputs/option";

import styles from "./booksSelector.module.scss";

/** Component to select on of many books
 * implements to views, a standard dropdown
 * and an frontpage view.
 */
export function BookSelector({
    books,
    currentIdx,
    setCurrentIdx,
}: {
    books: BookDataRet[];
    currentIdx: number;
    setCurrentIdx: Dispatch<SetStateAction<number>>;
}) {
    const [view, setView] = useState<"list" | "grid">("list");

    const [optionsList, optionsGrid] = useMemo(() => {
        const list: DropdownItem[] = [];
        const grid: DropdownItem[] = [];

        for (let i = 0; i < books.length; i++) {
            list.push({
                label: books[i]!.title,
                eventKey: books[i]!.id,
            });
            grid.push({
                label: books[i]!.title,
                labelSelect: <BookFrontpage book={books[i]!} />,
                eventKey: books[i]!.id,
            });
        }
        return [list, grid];
    }, [books]);

    return (
        <div className={styles.wrapper}>
            <DropdownInput
                value={books.length > 0 ? books[currentIdx]!.id : 0}
                label="Book"
                options={view == "list" ? optionsList : optionsGrid}
                onSelect={(id: string | null) => {
                    const idx = books.findIndex(
                        (book) => book.id === parseInt(id ?? ""),
                    );
                    setCurrentIdx(idx);
                }}
                menuProps={{
                    className:
                        styles.menu + " " + (view == "grid" ? styles.grid : ""),
                }}
            />
            <InputGroup className="w-auto">
                <Button
                    variant="outline-primary"
                    onClick={() => setView("list")}
                    active={view == "list"}
                >
                    <TbLayoutList />
                </Button>
                <Button
                    variant="outline-primary"
                    onClick={() => setView("grid")}
                    active={view == "grid"}
                >
                    <TbLayoutGrid />
                </Button>
            </InputGroup>
        </div>
    );
}

function BookFrontpage({ book }: { book: BookDataRet }) {
    return (
        <div className={styles.bookItem}>
            <label>{book.title}</label>
            <Image
                src={book.id.toString()}
                alt="Page"
                width={150}
                height={150 * 1.41}
                loader={({ src, width }) => {
                    // 200 hardcoded for now
                    return `/render/book_preview/${src}.jpeg?w=${width}&date=${book.last_updated?.valueOf()}`;
                }}
            />
        </div>
    );
}
