"use client";
import { usePersistentState } from "lib/hooks/usePersistentState";
import {
    createContext,
    Dispatch,
    SetStateAction,
    useContext,
    useEffect,
    useRef,
    useState,
} from "react";

import { useToolsContext } from "../context";
import { ToolContextProps } from "../types";

export interface DoodleContextI {
    // The current tool
    colorList: string[];
    setColorList: (colorList: string[]) => void;

    // Current color
    currentColor: string;
    currentColorIdx: number;
    setCurrentColorIdx: Dispatch<SetStateAction<number>>;
    // Pen size
    size: number;
    setSize: (size: number) => void;

    // Pen smoothing
    smoothing: number;
    setSmoothing: (smoothing: number) => void;

    // Touch events as move
    touchMove: boolean;
    setTouchMove: (touchMove: boolean) => void;
}

const DoodleToolContext = createContext<DoodleContextI | null>(null);

export function DoodleToolContextProvider({ children }: ToolContextProps) {
    const mounted = useRef(false);
    const { defaultUserConfig } = useToolsContext();

    const [colorList, setColorList] = useState([
        "#000000",
        "#0000ff",
        "#ff0000",
        "#00ff00",
    ]);
    const [currentColorIdx, setCurrentColorIdx] = useState(0);

    const [size, setSize] = useState(5);
    const [smoothing, setSmoothing] = useState(0.5);

    const [touchMove, setTouchMove] = usePersistentState("touchMove", false);

    // Sync local state from user defaults
    useEffect(() => {
        if (mounted.current || !defaultUserConfig) return;
        mounted.current = true;

        setColorList((prev) => {
            if (defaultUserConfig.pen_colors) {
                return defaultUserConfig.pen_colors;
            }
            return prev;
        });
        setCurrentColorIdx((prev) => {
            if (defaultUserConfig.pen_colors && defaultUserConfig.pen_color) {
                const idx = defaultUserConfig.pen_colors.indexOf(
                    defaultUserConfig.pen_color,
                );
                if (idx !== -1) return idx;
            }
            return prev;
        });

        setSize((prev) => {
            if (defaultUserConfig.pen_size) {
                return defaultUserConfig.pen_size;
            }
            return prev;
        });

        setSmoothing((prev) => {
            if (defaultUserConfig.pen_smoothing) {
                return defaultUserConfig.pen_smoothing;
            }
            return prev;
        });

        return () => {
            mounted.current = false;
        };
    }, [defaultUserConfig]);

    return (
        <DoodleToolContext.Provider
            value={{
                colorList,
                setColorList,
                currentColor: colorList[currentColorIdx]!,
                currentColorIdx,
                setCurrentColorIdx,
                size,
                setSize,
                smoothing,
                setSmoothing,
                touchMove,
                setTouchMove,
            }}
        >
            {children}
        </DoodleToolContext.Provider>
    );
}

export function useDoodleToolContext() {
    const context = useContext(DoodleToolContext);
    if (!context)
        throw new Error("Cant use DoodleToolContext outside provider!");
    return context;
}
